/* Color.h presents the declaration of class Color.
 *  ...
 *******************************************************************/

#ifndef COLOR
#define COLOR

#include <iostream.h>                           // istream, ostream

enum ColorValue {COLOR_UNDERFLOW = -1, 
                 RED, ORANGE, YELLOW, GREEN,
                 BLUE, INDIGO, VIOLET,
                 COLOR_OVERFLOW,
                 NUMBER_OF_COLORS = 7};
class Color
{
 public:
   Color();                                            // constructors
   Color(ColorValue initialColorValue);

   double Wavelength() const;                          // properties
   double Frequency() const;

   void Read(istream & in);                            // I/O
   void Print(ostream & out) const;
                                                       // relationals
   friend bool operator==(const Color & left, const Color & right);
   friend bool operator!=(const Color & left, const Color & right);
   friend bool operator<(const Color & left, const Color & right);
   friend bool operator>(const Color & left, const Color & right);
   friend bool operator<=(const Color & left, const Color & right);
   friend bool operator>=(const Color & left, const Color & right);

   Color operator++();                                 // prefix ++
   Color operator++(int);                              // postfix ++

   Color operator--();                                 // prefix --
   Color operator--(int);                              // postfix --

 private:
   ColorValue myColorValue;
};

// inline definitions 

// ------- Default Value Constructor -------------------------------------

inline Color::Color()
{
   myColorValue = RED;
}


// ------- Explicit Value Constructor ------------------------------------

#include <cassert>                                    // assert()

inline Color::Color(ColorValue initialColorValue)
{
   assert(RED <= initialColorValue && initialColorValue <= VIOLET);
   myColorValue = initialColorValue;
}


// -------- Non-member Input ---------------------------------------

inline istream & operator>>(istream & in, Color & theColor)
{
   theColor.Read(in);
   return in;
}


// -------- Non-member Output ---------------------------------------

inline ostream & operator<<(ostream & out, const Color & theColor)
{
   theColor.Print(out);
   return out;
 }


// -------- Equality -------------------------------------------------

inline bool operator==(const Color & left, const Color & right)
{
   return left.myColorValue == right.myColorValue;
}


// -------- Less-than ------------------------------------------------

inline bool operator<(const Color & left, const Color & right)
{
   return left.myColorValue < right.myColorValue;
}


// -------- Less-than-or-equal ---------------------------------------

inline bool operator<=(const Color & left, const Color & right)
{
   return left.myColorValue <= right.myColorValue;
}

// ... remaining relational operators omitted

#endif

